@@ -29,16 +29,18 @@
 #include <stdio.h>
 
 #include <glib.h>
+#include <ell/ell.h>
 
 #include <ofono/log.h>
 #include <ofono/modem.h>
 #include <ofono/netreg.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 
 #include "common.h"
-#include "atmodem.h"
 #include "vendor.h"
 
 #include "network-registration.h"
@@ -357,14 +359,9 @@ static void cops_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	if (g_at_result_iter_next_number(&iter, &tech) == FALSE)
 		tech = ACCESS_TECHNOLOGY_GSM;
 
-	strncpy(op.name, name, OFONO_MAX_OPERATOR_NAME_LENGTH);
-	op.name[OFONO_MAX_OPERATOR_NAME_LENGTH] = '\0';
-
-	strncpy(op.mcc, nd->mcc, OFONO_MAX_MCC_LENGTH);
-	op.mcc[OFONO_MAX_MCC_LENGTH] = '\0';
-
-	strncpy(op.mnc, nd->mnc, OFONO_MAX_MNC_LENGTH);
-	op.mnc[OFONO_MAX_MNC_LENGTH] = '\0';
+	l_strlcpy(op.name, name, sizeof(op.name));
+	l_strlcpy(op.mcc, nd->mcc, sizeof(op.mcc));
+	l_strlcpy(op.mnc, nd->mnc, sizeof(op.mnc));
 
 	/* Set to current */
 	op.status = 2;
@@ -1939,6 +1936,7 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 	switch (nd->vendor) {
 	case OFONO_VENDOR_SIMCOM:
+	case OFONO_VENDOR_SIMCOM_A76XX:
 		/* Register for CSQ changes */
 		g_at_chat_send(nd->chat, "AT+AUTOCSQ=1,1", none_prefix,
 				NULL, NULL, NULL);
@@ -2235,7 +2233,6 @@ void at_netreg_remove(struct ofono_netreg *netreg)
 }
 
 static const struct ofono_netreg_driver driver = {
-	.name				= "atmodem",
 	.probe				= at_netreg_probe,
 	.remove				= at_netreg_remove,
 	.registration_status		= at_registration_status,
@@ -2246,12 +2243,4 @@ static const struct ofono_netreg_driver driver = {
 	.strength			= at_signal_strength,
 };
 
-void at_netreg_init(void)
-{
-	ofono_netreg_driver_register(&driver);
-}
-
-void at_netreg_exit(void)
-{
-	ofono_netreg_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(netreg, atmodem, &driver)
