@@ -30,8 +30,6 @@
 #include <ofono/modem.h>
 #include <ofono/gprs-context.h>
 
-#include <ell/ell.h>
-
 #include "qmi.h"
 #include "wds.h"
 #include "util.h"
@@ -46,25 +44,30 @@ struct gprs_context_data {
 
 static void pkt_status_notify(struct qmi_result *result, void *user_data)
 {
+	static const uint8_t RESULT_CONN_STATUS = 0x01;
+	static const uint8_t RESULT_IP_FAMILY = 0x12;
 	struct ofono_gprs_context *gc = user_data;
 	struct gprs_context_data *data = ofono_gprs_context_get_data(gc);
-	const struct qmi_wds_notify_conn_status *status;
+	const struct {
+		uint8_t status;
+		uint8_t reconf;
+	} __attribute__((__packed__)) *status;
 	uint16_t len;
 	uint8_t ip_family;
 
 	DBG("");
 
-	status = qmi_result_get(result, QMI_WDS_NOTIFY_CONN_STATUS, &len);
+	status = qmi_result_get(result, RESULT_CONN_STATUS, &len);
 	if (!status)
 		return;
 
 	DBG("conn status %d", status->status);
 
-	if (qmi_result_get_uint8(result, QMI_WDS_NOTIFY_IP_FAMILY, &ip_family))
+	if (qmi_result_get_uint8(result, RESULT_IP_FAMILY, &ip_family))
 		DBG("ip family %d", ip_family);
 
 	switch (status->status) {
-	case QMI_WDS_CONN_STATUS_DISCONNECTED:
+	case QMI_WDS_CONNECTION_STATUS_DISCONNECTED:
 		if (data->pkt_handle) {
 			/* The context has been disconnected by the network */
 			ofono_gprs_context_deactivated(gc, data->active_context);
@@ -75,67 +78,105 @@ static void pkt_status_notify(struct qmi_result *result, void *user_data)
 	}
 }
 
-static void get_settings_cb(struct qmi_result *result, void *user_data)
+static void get_settings_ipv6(struct ofono_gprs_context *gc,
+					struct qmi_result *result)
 {
-	struct cb_data *cbd = user_data;
-	ofono_gprs_context_cb_t cb = cbd->cb;
-	struct ofono_gprs_context *gc = cbd->user;
-	uint8_t pdp_type, ip_family;
+	static const uint8_t RESULT_IP_ADDRESS = 0x25;
+	static const uint8_t RESULT_GATEWAY = 0x26;
+	static const uint8_t RESULT_PRIMARY_DNS = 0x27;
+	static const uint8_t RESULT_SECONDARY_DNS = 0x28;
+	static const uint8_t RESULT_MTU = 0x29;
+	const char *dns[3] = { NULL, NULL, NULL };
+	char dns1str[INET6_ADDRSTRLEN];
+	char dns2str[INET6_ADDRSTRLEN];
+	char ipv6str[INET6_ADDRSTRLEN];
+	const void *tlv;
+	uint16_t len;
+	uint32_t mtu;
+
+	tlv = qmi_result_get(result, RESULT_IP_ADDRESS, &len);
+	if (tlv && len == sizeof(struct in6_addr) + 1) {
+		const struct in6_addr *ip = tlv;
+		uint8_t prefix_len = l_get_u8(ip + 1);
+
+		inet_ntop(AF_INET6, ip, ipv6str, sizeof(ipv6str));
+		ofono_gprs_context_set_ipv6_address(gc, ipv6str);
+		ofono_gprs_context_set_ipv6_prefix_length(gc, prefix_len);
+	}
+
+	tlv = qmi_result_get(result, RESULT_GATEWAY, &len);
+	if (tlv && len == sizeof(struct in6_addr) + 1) {
+		const struct in6_addr *gw = tlv;
+
+		inet_ntop(AF_INET6, gw, ipv6str, sizeof(ipv6str));
+		ofono_gprs_context_set_ipv6_gateway(gc, ipv6str);
+	}
+
+	tlv = qmi_result_get(result, RESULT_PRIMARY_DNS, &len);
+	if (tlv && len == sizeof(struct in6_addr)) {
+		const struct in6_addr *dns1 = tlv;
+
+		inet_ntop(AF_INET6, dns1, dns1str, sizeof(dns1str));
+		dns[0] = dns1str;
+	}
+
+	tlv = qmi_result_get(result, RESULT_SECONDARY_DNS, &len);
+	if (tlv && len == sizeof(struct in6_addr)) {
+		const struct in6_addr *dns2 = tlv;
+
+		inet_ntop(AF_INET6, dns2, dns2str, sizeof(dns2str));
+		dns[1] = dns2str;
+	}
+
+	if (dns[0])
+		ofono_gprs_context_set_ipv6_dns_servers(gc, dns);
+
+	if (qmi_result_get_uint32(result, RESULT_MTU, &mtu))
+		DBG("MTU: %u", mtu);
+}
+
+static void get_settings_ipv4(struct ofono_gprs_context *gc,
+					struct qmi_result *result)
+{
+	static const uint8_t RESULT_PRIMARY_DNS = 0x15;
+	static const uint8_t RESULT_SECONDARY_DNS = 0x16;
+	static const uint8_t RESULT_IP_ADDRESS = 0x1e;
+	static const uint8_t RESULT_GATEWAY = 0x20;
+	static const uint8_t RESULT_GATEWAY_NETMASK = 0x21;
 	uint32_t ip_addr;
 	struct in_addr addr;
 	char* straddr;
-	char* apn;
 	const char *dns[3] = { NULL, NULL, NULL };
 	char dns_buf[2][INET_ADDRSTRLEN];
 
-	DBG("");
-
-	if (qmi_result_set_error(result, NULL))
-		goto done;
-
-	apn = qmi_result_get_string(result, QMI_WDS_RESULT_APN);
-	if (apn) {
-		DBG("APN: %s", apn);
-		g_free(apn);
-	}
-
-	if (qmi_result_get_uint8(result, QMI_WDS_RESULT_PDP_TYPE, &pdp_type))
-		DBG("PDP type %d", pdp_type);
-
-	if (qmi_result_get_uint8(result, QMI_WDS_RESULT_IP_FAMILY, &ip_family))
-		DBG("IP family %d", ip_family);
-
-	if (qmi_result_get_uint32(result,QMI_WDS_RESULT_IP_ADDRESS, &ip_addr)) {
+	if (qmi_result_get_uint32(result, RESULT_IP_ADDRESS, &ip_addr)) {
 		addr.s_addr = htonl(ip_addr);
 		straddr = inet_ntoa(addr);
 		DBG("IP addr: %s", straddr);
 		ofono_gprs_context_set_ipv4_address(gc, straddr, 1);
 	}
 
-	if (qmi_result_get_uint32(result,QMI_WDS_RESULT_GATEWAY, &ip_addr)) {
+	if (qmi_result_get_uint32(result, RESULT_GATEWAY, &ip_addr)) {
 		addr.s_addr = htonl(ip_addr);
 		straddr = inet_ntoa(addr);
 		DBG("Gateway: %s", straddr);
 		ofono_gprs_context_set_ipv4_gateway(gc, straddr);
 	}
 
-	if (qmi_result_get_uint32(result,
-				QMI_WDS_RESULT_GATEWAY_NETMASK, &ip_addr)) {
+	if (qmi_result_get_uint32(result, RESULT_GATEWAY_NETMASK, &ip_addr)) {
 		addr.s_addr = htonl(ip_addr);
 		straddr = inet_ntoa(addr);
 		DBG("Gateway netmask: %s", straddr);
 		ofono_gprs_context_set_ipv4_netmask(gc, straddr);
 	}
 
-	if (qmi_result_get_uint32(result,
-				QMI_WDS_RESULT_PRIMARY_DNS, &ip_addr)) {
+	if (qmi_result_get_uint32(result, RESULT_PRIMARY_DNS, &ip_addr)) {
 		addr.s_addr = htonl(ip_addr);
 		dns[0] = inet_ntop(AF_INET, &addr, dns_buf[0], sizeof(dns_buf[0]));
 		DBG("Primary DNS: %s", dns[0]);
 	}
 
-	if (qmi_result_get_uint32(result,
-				QMI_WDS_RESULT_SECONDARY_DNS, &ip_addr)) {
+	if (qmi_result_get_uint32(result, RESULT_SECONDARY_DNS, &ip_addr)) {
 		addr.s_addr = htonl(ip_addr);
 		dns[1] = inet_ntop(AF_INET, &addr, dns_buf[1], sizeof(dns_buf[1]));
 		DBG("Secondary DNS: %s", dns[1]);
@@ -143,6 +184,48 @@ static void get_settings_cb(struct qmi_result *result, void *user_data)
 
 	if (dns[0])
 		ofono_gprs_context_set_ipv4_dns_servers(gc, dns);
+}
+
+static void get_settings_cb(struct qmi_result *result, void *user_data)
+{
+	static const uint8_t RESULT_PDP_TYPE = 0x11;	/* uint8 */
+	static const uint8_t RESULT_APN = 0x14;		/* string */
+	static const uint8_t RESULT_IP_FAMILY = 0x2b;	/* uint8 */
+	struct cb_data *cbd = user_data;
+	ofono_gprs_context_cb_t cb = cbd->cb;
+	struct ofono_gprs_context *gc = cbd->user;
+	uint8_t pdp_type, ip_family;
+	char *apn;
+
+	DBG("");
+
+	if (qmi_result_set_error(result, NULL))
+		goto done;
+
+	apn = qmi_result_get_string(result, RESULT_APN);
+	if (apn) {
+		DBG("APN: %s", apn);
+		l_free(apn);
+	}
+
+	if (qmi_result_get_uint8(result, RESULT_PDP_TYPE, &pdp_type))
+		DBG("PDP type %d", pdp_type);
+
+	if (!qmi_result_get_uint8(result, RESULT_IP_FAMILY, &ip_family)) {
+		ofono_error("No IP family in results");
+		goto done;
+	}
+
+	switch (ip_family) {
+	case QMI_WDS_IP_FAMILY_IPV4:
+		get_settings_ipv4(gc, result);
+		break;
+	case QMI_WDS_IP_FAMILY_IPV6:
+		get_settings_ipv6(gc, result);
+		break;
+	default:
+		break;
+	}
 
 done:
 	CALLBACK_WITH_SUCCESS(cb, cbd->data);
@@ -150,35 +233,98 @@ done:
 
 static void start_net_cb(struct qmi_result *result, void *user_data)
 {
+	static const uint8_t RESULT_PACKET_HANDLE = 0x01;
+	static const uint8_t PARAM_REQUESTED_SETTINGS = 0x10;
 	struct cb_data *cbd = user_data;
 	ofono_gprs_context_cb_t cb = cbd->cb;
 	struct ofono_gprs_context *gc = cbd->user;
 	struct gprs_context_data *data = ofono_gprs_context_get_data(gc);
 	uint32_t handle;
+	uint32_t requested_settings = 0;
+	struct qmi_param *param;
 
 	DBG("");
 
 	if (qmi_result_set_error(result, NULL))
 		goto error;
 
-	if (!qmi_result_get_uint32(result, QMI_WDS_RESULT_PKT_HANDLE, &handle))
+	if (!qmi_result_get_uint32(result, RESULT_PACKET_HANDLE, &handle))
 		goto error;
 
 	DBG("packet handle %d", handle);
 
 	data->pkt_handle = handle;
 
-	/* Duplicate cbd, the old one will be freed when this method returns */
-	cbd = cb_data_new(cb, cbd->data);
-	cbd->user = gc;
+	/*
+	 * Explicitly request certain information to be provided.  The requested
+	 * settings is a bit field, with each bit representing whether the
+	 * TLV is included in the GET_CURRENT_SETTINGS response.  We request the
+	 * following settings:
+	 * 2 - PDP Type, 3 - APN Name, 4 - DNS, 5 - Granted QOS,
+	 * 6 - Username, 7 - Auth Proto
+	 * 8 - IP Address, 9 - Gateway, 13 - MTU, 14 - DNS List,
+	 * 15 - IP Family, 17 - Extended Technology
+	 */
+	L_BITS_SET(&requested_settings, 2, 3, 4, 5, 6, 7, 8, 9, 13, 14, 15, 17);
+	param = qmi_param_new_uint32(PARAM_REQUESTED_SETTINGS,
+						requested_settings);
 
-	if (qmi_service_send(data->wds, QMI_WDS_GET_CURRENT_SETTINGS, NULL,
-					get_settings_cb, cbd, g_free) > 0)
+	if (qmi_service_send(data->wds, QMI_WDS_GET_CURRENT_SETTINGS, param,
+				get_settings_cb, cbd, cb_data_unref) > 0) {
+		cb_data_ref(cbd);
 		return;
+	}
 
-	CALLBACK_WITH_SUCCESS(cb, cbd->data);
+	qmi_param_free(param);
 
-	return;
+error:
+	data->active_context = 0;
+	CALLBACK_WITH_FAILURE(cb, cbd->data);
+}
+
+static void get_lte_attach_param_cb(struct qmi_result *result, void *user_data)
+{
+	static const uint8_t RESULT_IP_SUPPORT_TYPE = 0x11;
+	struct cb_data *cbd = user_data;
+	ofono_gprs_context_cb_t cb = cbd->cb;
+	struct ofono_gprs_context *gc = cbd->user;
+	struct gprs_context_data *data = ofono_gprs_context_get_data(gc);
+	uint16_t error;
+	uint8_t iptype;
+	struct qmi_param *param;
+	uint8_t ip_family;
+
+	DBG("");
+
+	if (qmi_result_set_error(result, &error))
+		goto error;
+
+	if (!qmi_result_get_uint8(result, RESULT_IP_SUPPORT_TYPE, &iptype))
+		goto error;
+
+	switch (iptype) {
+	case QMI_WDS_IP_SUPPORT_IPV4:
+		ip_family = QMI_WDS_IP_FAMILY_IPV4;
+		break;
+	case QMI_WDS_IP_SUPPORT_IPV6:
+		ip_family = QMI_WDS_IP_FAMILY_IPV6;
+		break;
+	case QMI_WDS_IP_SUPPORT_IPV4V6:
+		ip_family = QMI_WDS_IP_FAMILY_IPV4;
+		break;
+	default:
+		goto error;
+	}
+
+	param = qmi_param_new_uint8(QMI_WDS_PARAM_IP_FAMILY, ip_family);
+
+	if (qmi_service_send(data->wds, QMI_WDS_START_NETWORK, param,
+					start_net_cb, cbd, cb_data_unref) > 0) {
+		cb_data_ref(cbd);
+		return;
+	}
+
+	qmi_param_free(param);
 
 error:
 	data->active_context = 0;
@@ -202,33 +348,16 @@ static void qmi_gprs_read_settings(struct ofono_gprs_context* gc,
 
 	DBG("cid %u", cid);
 
-	data->active_context = cid;
-
-	cbd->user = gc;
-
-	if (qmi_service_send(data->wds, QMI_WDS_START_NETWORK, NULL,
-					start_net_cb, cbd, g_free) > 0)
+	if (qmi_service_send(data->wds, QMI_WDS_GET_LTE_ATTACH_PARAMETERS,
+				NULL, get_lte_attach_param_cb, cbd,
+				cb_data_unref) > 0) {
+		data->active_context = cid;
+		cbd->user = gc;
 		return;
-
-	data->active_context = 0;
-
-	CALLBACK_WITH_FAILURE(cb, cbd->data);
-
-	g_free(cbd);
-}
-
-static uint8_t auth_method_to_qmi_auth(enum ofono_gprs_auth_method method)
-{
-	switch (method) {
-	case OFONO_GPRS_AUTH_METHOD_CHAP:
-		return QMI_WDS_AUTHENTICATION_CHAP;
-	case OFONO_GPRS_AUTH_METHOD_PAP:
-		return QMI_WDS_AUTHENTICATION_PAP;
-	case OFONO_GPRS_AUTH_METHOD_NONE:
-		return QMI_WDS_AUTHENTICATION_NONE;
 	}
 
-	return QMI_WDS_AUTHENTICATION_NONE;
+	CALLBACK_WITH_FAILURE(cb, cbd->data);
+	l_free(cbd);
 }
 
 static void qmi_activate_primary(struct ofono_gprs_context *gc,
@@ -249,10 +378,10 @@ static void qmi_activate_primary(struct ofono_gprs_context *gc,
 
 	switch (ctx->proto) {
 	case OFONO_GPRS_PROTO_IP:
-		ip_family = 4;
+		ip_family = QMI_WDS_IP_FAMILY_IPV4;
 		break;
 	case OFONO_GPRS_PROTO_IPV6:
-		ip_family = 6;
+		ip_family = QMI_WDS_IP_FAMILY_IPV6;
 		break;
 	default:
 		goto error;
@@ -265,21 +394,21 @@ static void qmi_activate_primary(struct ofono_gprs_context *gc,
 
 	qmi_param_append_uint8(param, QMI_WDS_PARAM_IP_FAMILY, ip_family);
 
-	auth = auth_method_to_qmi_auth(ctx->auth_method);
+	auth = qmi_wds_auth_from_ofono(ctx->auth_method);
 
 	qmi_param_append_uint8(param, QMI_WDS_PARAM_AUTHENTICATION_PREFERENCE,
 					auth);
 
-	if (auth != QMI_WDS_AUTHENTICATION_NONE && ctx->username[0] != '\0')
+	if (auth && ctx->username[0] != '\0')
 		qmi_param_append(param, QMI_WDS_PARAM_USERNAME,
 					strlen(ctx->username), ctx->username);
 
-	if (auth != QMI_WDS_AUTHENTICATION_NONE &&  ctx->password[0] != '\0')
+	if (auth && ctx->password[0] != '\0')
 		qmi_param_append(param, QMI_WDS_PARAM_PASSWORD,
 					strlen(ctx->password), ctx->password);
 
 	if (qmi_service_send(data->wds, QMI_WDS_START_NETWORK, param,
-					start_net_cb, cbd, g_free) > 0)
+					start_net_cb, cbd, cb_data_unref) > 0)
 		return;
 
 	qmi_param_free(param);
@@ -289,7 +418,7 @@ error:
 
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
 
-	g_free(cbd);
+	l_free(cbd);
 }
 
 static void stop_net_cb(struct qmi_result *result, void *user_data)
@@ -321,6 +450,7 @@ static void qmi_deactivate_primary(struct ofono_gprs_context *gc,
 				unsigned int cid,
 				ofono_gprs_context_cb_t cb, void *user_data)
 {
+	static const uint8_t PARAM_PACKET_HANDLE = 0x01;
 	struct gprs_context_data *data = ofono_gprs_context_get_data(gc);
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 	struct qmi_param *param;
@@ -329,11 +459,10 @@ static void qmi_deactivate_primary(struct ofono_gprs_context *gc,
 
 	cbd->user = gc;
 
-	param = qmi_param_new_uint32(QMI_WDS_PARAM_PKT_HANDLE,
-						data->pkt_handle);
+	param = qmi_param_new_uint32(PARAM_PACKET_HANDLE, data->pkt_handle);
 
 	if (qmi_service_send(data->wds, QMI_WDS_STOP_NETWORK, param,
-					stop_net_cb, cbd, g_free) > 0)
+					stop_net_cb, cbd, l_free) > 0)
 		return;
 
 	qmi_param_free(param);
@@ -341,7 +470,7 @@ static void qmi_deactivate_primary(struct ofono_gprs_context *gc,
 	if (cb)
 		CALLBACK_WITH_FAILURE(cb, user_data);
 
-	g_free(cbd);
+	l_free(cbd);
 }
 
 static void qmi_gprs_context_detach_shutdown(struct ofono_gprs_context *gc,
@@ -378,21 +507,6 @@ static void qmi_gprs_context_bind_mux(struct ofono_gprs_context *gc)
 	} __attribute__((packed)) endpoint_info;
 	uint8_t u8;
 
-	interface_number = ofono_modem_get_string(modem, "InterfaceNumber");
-	if (!interface_number) {
-		ofono_error("%s: Missing 'InterfaceNumber'",
-					ofono_modem_get_path(modem));
-		goto error;
-	}
-
-	if (l_safe_atox8(interface_number, &u8) < 0) {
-		ofono_error("%s: Invalid InterfaceNumber",
-					ofono_modem_get_path(modem));
-		goto error;
-	}
-
-	endpoint_info.interface_number = u8;
-
 	bus = ofono_modem_get_string(modem, "Bus");
 	if (!bus) {
 		ofono_error("%s: Missing 'Bus'", ofono_modem_get_path(modem));
@@ -411,6 +525,23 @@ static void qmi_gprs_context_bind_mux(struct ofono_gprs_context *gc)
 		goto error;
 	}
 
+	interface_number = ofono_modem_get_string(modem, "InterfaceNumber");
+	if (!interface_number && endpoint_info.endpoint_type !=
+					QMI_DATA_ENDPOINT_TYPE_EMBEDDED) {
+		ofono_error("%s: Missing 'InterfaceNumber'",
+					ofono_modem_get_path(modem));
+		goto error;
+	} else if (!interface_number)
+		u8 = 1;	/* Default for embedded modems */
+	else if (l_safe_atox8(interface_number, &u8) < 0) {
+		ofono_error("%s: Invalid InterfaceNumber",
+					ofono_modem_get_path(modem));
+		goto error;
+	}
+
+	endpoint_info.interface_number = u8;
+
+	DBG("interface_number: %d", u8);
 	DBG("mux_id: %hhx", data->mux_id);
 
 	param = qmi_param_new();
@@ -442,7 +573,7 @@ static void create_wds_cb(struct qmi_service *service, void *user_data)
 		return;
 	}
 
-	data->wds = qmi_service_ref(service);
+	data->wds = service;
 
 	qmi_service_register(data->wds, QMI_WDS_PACKET_SERVICE_STATUS,
 					pkt_status_notify, gc, NULL);
@@ -459,7 +590,7 @@ static int qmi_gprs_context_probe(struct ofono_gprs_context *gc,
 
 	DBG("");
 
-	data = g_new0(struct gprs_context_data, 1);
+	data = l_new(struct gprs_context_data, 1);
 
 	ofono_gprs_context_set_data(gc, data);
 	data->dev = device;
@@ -478,12 +609,10 @@ static void qmi_gprs_context_remove(struct ofono_gprs_context *gc)
 
 	ofono_gprs_context_set_data(gc, NULL);
 
-	if (data->wds) {
-		qmi_service_unregister_all(data->wds);
-		qmi_service_unref(data->wds);
-	}
+	if (data->wds)
+		qmi_service_free(data->wds);
 
-	g_free(data);
+	l_free(data);
 }
 
 static const struct ofono_gprs_context_driver driver = {
