@@ -38,8 +38,7 @@
 
 #include "qmi.h"
 #include "wds.h"
-
-#include "qmimodem.h"
+#include "util.h"
 
 struct lte_data {
 	struct qmi_service *wds;
@@ -67,6 +66,10 @@ static void qmimodem_lte_set_default_attach_info(const struct ofono_lte *lte,
 			const struct ofono_lte_default_attach_info *info,
 			ofono_lte_cb_t cb, void *data)
 {
+	static const uint8_t PARAM_PDP_TYPE = 0x11;
+	static const uint8_t PARAM_USERNAME = 0x1B;
+	static const uint8_t PARAM_PASSWORD = 0x1C;
+	static const uint8_t PARAM_AUTHENTICATION_PREFERENCE = 0x1D;
 	struct lte_data *ldd = ofono_lte_get_data(lte);
 	struct cb_data *cbd = cb_data_new(cb, data);
 	struct qmi_param* param;
@@ -74,33 +77,38 @@ static void qmimodem_lte_set_default_attach_info(const struct ofono_lte *lte,
 		uint8_t type;
 		uint8_t index;
 	} __attribute__((packed)) p = {
-		.type = 0, /* 3GPP */
+		.type = QMI_WDS_PROFILE_TYPE_3GPP,
+		.index = ldd->default_profile,
 	};
+	uint8_t auth = qmi_wds_auth_from_ofono(info->auth_method);
 
 	DBG("");
 
-	p.index = ldd->default_profile;
-
 	param = qmi_param_new();
-	if (!param)
-		goto error;
 
-	/* Profile selector */
-	qmi_param_append(param, 0x01, sizeof(p), &p);
-
-	/* WDS APN Name */
+	qmi_param_append(param, QMI_WDS_PARAM_PROFILE_TYPE, sizeof(p), &p);
+	qmi_param_append_uint8(param, PARAM_PDP_TYPE,
+				qmi_wds_pdp_type_from_ofono(info->proto));
 	qmi_param_append(param, QMI_WDS_PARAM_APN,
 				strlen(info->apn), info->apn);
 
-	/* Modify profile */
-	if (qmi_service_send(ldd->wds, 0x28, param,
-					modify_profile_cb, cbd, g_free) > 0)
+	qmi_param_append_uint8(param, PARAM_AUTHENTICATION_PREFERENCE, auth);
+
+	if (auth && info->username[0])
+		qmi_param_append(param, PARAM_USERNAME,
+					strlen(info->username), info->username);
+
+	if (auth && info->password[0])
+		qmi_param_append(param, PARAM_PASSWORD,
+					strlen(info->password), info->password);
+
+	if (qmi_service_send(ldd->wds, QMI_WDS_MODIFY_PROFILE, param,
+					modify_profile_cb, cbd, l_free) > 0)
 		return;
 
 	qmi_param_free(param);
-
-error:
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
+	l_free(cbd);
 }
 
 static void reset_profile_cb(struct qmi_result *result, void *user_data)
@@ -118,6 +126,7 @@ static void reset_profile_cb(struct qmi_result *result, void *user_data)
 
 static void get_default_profile_cb(struct qmi_result *result, void *user_data)
 {
+	static const uint8_t RESULT_DEFAULT_PROFILE_NUMBER = 0x1;
 	struct ofono_lte *lte = user_data;
 	struct lte_data *ldd = ofono_lte_get_data(lte);
 	uint16_t error;
@@ -127,7 +136,7 @@ static void get_default_profile_cb(struct qmi_result *result, void *user_data)
 		uint8_t type;
 		uint8_t index;
 	} __attribute__((packed)) p = {
-		.type = 0, /* 3GPP */
+		.type = QMI_WDS_PROFILE_TYPE_3GPP,
 	};
 
 	DBG("");
@@ -138,7 +147,8 @@ static void get_default_profile_cb(struct qmi_result *result, void *user_data)
 	}
 
 	/* Profile index */
-	if (!qmi_result_get_uint8(result, 0x01, &index)) {
+	if (!qmi_result_get_uint8(result, RESULT_DEFAULT_PROFILE_NUMBER,
+								&index)) {
 		ofono_error("Failed query default profile");
 		goto error;
 	}
@@ -150,14 +160,12 @@ static void get_default_profile_cb(struct qmi_result *result, void *user_data)
 	p.index = index;
 
 	param = qmi_param_new();
-	if (!param)
-		goto error;
 
 	/* Profile selector */
-	qmi_param_append(param, 0x01, sizeof(p), &p);
+	qmi_param_append(param, QMI_WDS_PARAM_PROFILE_TYPE, sizeof(p), &p);
 
 	/* Reset profile */
-	if (qmi_service_send(ldd->wds, 0x4b, param,
+	if (qmi_service_send(ldd->wds, QMI_WDS_RESET_PROFILE, param,
 				reset_profile_cb, lte, NULL) > 0)
 		return;
 
@@ -177,8 +185,8 @@ static void create_wds_cb(struct qmi_service *service, void *user_data)
 		uint8_t type;
 		uint8_t family;
 	} __attribute((packed)) p = {
-		.type = 0,   /* 3GPP */
-		.family = 0, /* embedded */
+		.type = QMI_WDS_PROFILE_TYPE_3GPP,
+		.family = QMI_WDS_PROFILE_FAMILY_EMBEDDED,
 	};
 
 	DBG("");
@@ -189,24 +197,21 @@ static void create_wds_cb(struct qmi_service *service, void *user_data)
 		return;
 	}
 
-	ldd->wds = qmi_service_ref(service);
+	ldd->wds = service;
 
 	/* Query the default profile */
 	param = qmi_param_new();
-	if (!param)
-		goto error;
 
 	/* Profile type */
-	qmi_param_append(param, 0x1, sizeof(p), &p);
+	qmi_param_append(param, QMI_WDS_PARAM_PROFILE_TYPE, sizeof(p), &p);
 
 	/* Get default profile */
-	if (qmi_service_send(ldd->wds, 0x49, param,
-				get_default_profile_cb, lte, NULL) > 0)
+	if (qmi_service_send(ldd->wds, QMI_WDS_GET_DEFAULT_PROFILE_NUMBER,
+				param, get_default_profile_cb, lte, NULL) > 0)
 		return;
 
 	qmi_param_free(param);
 
-error:
 	ofono_error("Failed to query default profile");
 	ofono_lte_register(lte);
 }
@@ -219,9 +224,7 @@ static int qmimodem_lte_probe(struct ofono_lte *lte,
 
 	DBG("qmimodem lte probe");
 
-	ldd = g_try_new0(struct lte_data, 1);
-	if (!ldd)
-		return -ENOMEM;
+	ldd = l_new(struct lte_data, 1);
 
 	ofono_lte_set_data(lte, ldd);
 
@@ -239,26 +242,15 @@ static void qmimodem_lte_remove(struct ofono_lte *lte)
 
 	ofono_lte_set_data(lte, NULL);
 
-	qmi_service_unregister_all(ldd->wds);
+	qmi_service_free(ldd->wds);
 
-	qmi_service_unref(ldd->wds);
-
-	g_free(ldd);
+	l_free(ldd);
 }
 
 static const struct ofono_lte_driver driver = {
-	.name				= "qmimodem",
 	.probe				= qmimodem_lte_probe,
 	.remove				= qmimodem_lte_remove,
 	.set_default_attach_info	= qmimodem_lte_set_default_attach_info,
 };
 
-void qmi_lte_init(void)
-{
-	ofono_lte_driver_register(&driver);
-}
-
-void qmi_lte_exit(void)
-{
-	ofono_lte_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(lte, qmimodem, &driver)
