@@ -29,17 +29,15 @@
 #include <string.h>
 #include <errno.h>
 
-#include <glib.h>
-
 #include <ofono/log.h>
 #include <ofono/modem.h>
 #include <ofono/netmon.h>
 
+#include "src/common.h"
+
 #include "qmi.h"
 #include "nas.h"
-
-#include "qmimodem.h"
-#include "src/common.h"
+#include "util.h"
 
 struct netmon_data {
 	struct qmi_service *nas;
@@ -90,7 +88,7 @@ static void get_rssi_cb(struct qmi_result *result, void *user_data)
 	/* RSSI */
 	rssi = qmi_result_get(result, 0x11, &len);
 	if (rssi) {
-		num = GUINT16_FROM_LE(rssi->count);
+		num = L_LE16_TO_CPU(rssi->count);
 		for (i = 0; i < num; i++) {
 			DBG("RSSI: %hhu on RAT %hhd",
 				rssi->info[i].rssi,
@@ -127,14 +125,14 @@ static void get_rssi_cb(struct qmi_result *result, void *user_data)
 	/* Bit error rate */
 	ber = qmi_result_get(result, 0x15, &len);
 	if (ber) {
-		num = GUINT16_FROM_LE(ber->count);
+		num = L_LE16_TO_CPU(ber->count);
 		for (i = 0; i < ber->count; i++) {
 			DBG("Bit error rate: %hu on RAT %hhd",
-				GUINT16_FROM_LE(ber->info[i].rate),
+				L_LE16_TO_CPU(ber->info[i].rate),
 				ber->info[i].rat);
 		}
 
-		props.ber = GUINT16_FROM_LE(ber->info[0].rate);
+		props.ber = L_LE16_TO_CPU(ber->info[0].rate);
 		if (props.ber > 7)
 			props.ber = -1;
 	} else {
@@ -198,22 +196,17 @@ static void qmi_netmon_request_update(struct ofono_netmon *netmon,
 	cbd->user = netmon;
 
 	param = qmi_param_new();
-	if (!param)
-		goto out;
 
 	/* Request all signal strength items: mask=0xff */
 	qmi_param_append_uint16(param, 0x10, 255);
 
-	if (qmi_service_send(data->nas, QMI_NAS_GET_RSSI, param,
-					get_rssi_cb, cbd, g_free) > 0)
+	if (qmi_service_send(data->nas, QMI_NAS_GET_SIGNAL_STRENGTH, param,
+					get_rssi_cb, cbd, l_free) > 0)
 		return;
 
 	qmi_param_free(param);
-
-out:
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
-
-	g_free(cbd);
+	l_free(cbd);
 }
 
 static void create_nas_cb(struct qmi_service *service, void *user_data)
@@ -229,7 +222,7 @@ static void create_nas_cb(struct qmi_service *service, void *user_data)
 		return;
 	}
 
-	nmd->nas = qmi_service_ref(service);
+	nmd->nas = service;
 
 	ofono_netmon_register(netmon);
 }
@@ -242,7 +235,7 @@ static int qmi_netmon_probe(struct ofono_netmon *netmon,
 
 	DBG("");
 
-	nmd = g_new0(struct netmon_data, 1);
+	nmd = l_new(struct netmon_data, 1);
 
 	ofono_netmon_set_data(netmon, nmd);
 
@@ -260,26 +253,15 @@ static void qmi_netmon_remove(struct ofono_netmon *netmon)
 
 	ofono_netmon_set_data(netmon, NULL);
 
-	qmi_service_unregister_all(nmd->nas);
+	qmi_service_free(nmd->nas);
 
-	qmi_service_unref(nmd->nas);
-
-	g_free(nmd);
+	l_free(nmd);
 }
 
 static const struct ofono_netmon_driver driver = {
-	.name			= "qmimodem",
 	.probe			= qmi_netmon_probe,
 	.remove			= qmi_netmon_remove,
 	.request_update		= qmi_netmon_request_update,
 };
 
-void qmi_netmon_init(void)
-{
-	ofono_netmon_driver_register(&driver);
-}
-
-void qmi_netmon_exit(void)
-{
-	ofono_netmon_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(netmon, qmimodem, &driver)
