@@ -834,7 +834,8 @@ static gboolean setup_telitqmi(struct modem_info *modem)
 		DBG("%s %s %s %s %s", info->devnode, info->interface,
 				info->number, info->label, subsystem);
 
-		if (g_strcmp0(info->interface, "255/255/255") == 0 &&
+		if ((!g_strcmp0(info->interface, "255/255/255") ||
+				!g_strcmp0(info->interface, "255/255/80")) &&
 				g_strcmp0(info->number, "02") == 0) {
 			if (g_strcmp0(subsystem, "net") == 0)
 				net = info;
@@ -849,8 +850,8 @@ static gboolean setup_telitqmi(struct modem_info *modem)
 	if (setup_qmi(modem, qmi, net) < 0)
 		return FALSE;
 
-	ofono_modem_set_boolean(modem->modem, "ForceSimLegacy", TRUE);
-	ofono_modem_set_boolean(modem->modem, "AlwaysOnline", TRUE);
+	if (g_strcmp0(modem->model, "1070"))
+		ofono_modem_set_boolean(modem->modem, "ForceSimLegacy", TRUE);
 
 	return TRUE;
 }
@@ -1630,6 +1631,57 @@ static gboolean setup_sim7x00(struct modem_info *modem)
 	return TRUE;
 }
 
+static gboolean setup_sim76xx(struct modem_info *modem)
+{
+	const char *diag = NULL;
+	const char *mdm = NULL;
+	const char *ppp = NULL;
+	const char *gps = NULL;
+	GSList *list;
+
+	DBG("%s", modem->syspath);
+
+	for (list = modem->devices; list; list = list->next) {
+		const struct device_info *info = list->data;
+		const char *subsystem;
+
+		subsystem = udev_device_get_subsystem(info->udev_device);
+		if (!g_str_equal(subsystem, "tty"))
+			continue;
+
+		DBG("%s %s %s", info->devnode, info->interface, info->number);
+
+		/*
+		 * SIM76xx USB numbering:
+		 * 0: RNDIS (ep_87)
+		 * 1: RNDIS (ep_0c and ep_83)
+		 * 2: QCDM/DIAG (ttyUSB0)
+		 * 3: NMEA (ttyUSB3)
+		 * 4: AT (ttyUSB1)
+		 * 5: AT/PPP (ttyUSB2)
+		 */
+		if (g_str_equal(info->number, "02"))
+			diag = info->devnode;
+		else if (g_str_equal(info->number, "03"))
+			gps = info->devnode;
+		else if (g_str_equal(info->number, "04"))
+			mdm = info->devnode;
+		else if (g_str_equal(info->number, "05"))
+			ppp = info->devnode;
+	}
+
+	if (mdm == NULL)
+		return FALSE;
+
+	DBG("at=%s ppp=%s gps=%s diag=%s", mdm, ppp, gps, diag);
+
+	ofono_modem_set_driver(modem->modem, "sim7100");
+	ofono_modem_set_string(modem->modem, "AT", mdm);
+	ofono_modem_set_string(modem->modem, "PPP", ppp);
+
+	return TRUE;
+}
+
 static struct {
 	const char *name;
 	gboolean (*setup)(struct modem_info *modem);
@@ -1650,6 +1702,7 @@ static struct {
 	{ "telitqmi",	setup_telitqmi	},
 	{ "simcom",	setup_simcom	},
 	{ "sim7x00",	setup_sim7x00	},
+	{ "sim76xx",	setup_sim76xx	},
 	{ "zte",	setup_zte	},
 	{ "icera",	setup_icera	},
 	{ "samsung",	setup_samsung	},
@@ -2014,6 +2067,7 @@ static struct {
 	{ "hso",	"hso"				},
 	{ "gobi",	"qmi_wwan"			},
 	{ "gobi",	"qcserial"			},
+	{ "gobi",	"option"			},
 	{ "sierra",	"qmi_wwan",	"1199"		},
 	{ "sierra",	"qcserial",	"1199"		},
 	{ "sierra",	"sierra"			},
@@ -2032,11 +2086,14 @@ static struct {
 	{ "simcom",	"option",	"05c6", "9000"	},
 	{ "sim7x00",	"option",	"1e0e", "9001"	},
 	{ "sim7x00",	"qmi_wwan",	"1e0e",	"9001"	},
+	{ "sim76xx",	"option",	"1e0e", "9011"	},
 	{ "telit",	"usbserial",	"1bc7"		},
 	{ "telit",	"option",	"1bc7"		},
 	{ "telit",	"cdc_acm",	"1bc7", "0021"	},
 	{ "telitqmi",	"qmi_wwan",	"1bc7", "1201"	},
 	{ "telitqmi",	"option",	"1bc7", "1201"	},
+	{ "telitqmi",	"qmi_wwan",	"1bc7", "1070"	},
+	{ "telitqmi",	"option",	"1bc7", "1070"	},
 	{ "droid",	"qmi_wwan",	"22b8", "2a70"	},
 	{ "droid",	"option",	"22b8", "2a70"	},
 	{ "nokia",	"option",	"0421", "060e"	},
@@ -2052,6 +2109,9 @@ static struct {
 	{ "quectelqmi",	"qcserial",	"2c7c", "0195"	},
 	{ "quectelqmi",	"qmi_wwan",	"2c7c", "0296"	},
 	{ "quectelqmi",	"qcserial",	"2c7c", "0296"	},
+	{ "quectelqmi", "qmi_wwan",	"2c7c", "0800"	},
+	{ "quectelqmi", "qcserial",	"2c7c", "0800"	},
+	{ "quectelqmi", "option",	"2c7c", "0800"	},
 	{ "quectelqmi", "qmi_wwan_q",	"2c7c", "0452"	},
 	{ "ublox",	"cdc_acm",	"1546", "1010"	},
 	{ "ublox",	"cdc_ncm",	"1546", "1010"	},
