@@ -41,8 +41,6 @@
 #define SETTINGS_STORE "cbs"
 #define SETTINGS_GROUP "Settings"
 
-static GSList *g_drivers = NULL;
-
 enum etws_topic_type {
 	ETWS_TOPIC_TYPE_EARTHQUAKE =		4352,
 	ETWS_TOPIC_TYPE_TSUNAMI =		4353,
@@ -272,8 +270,7 @@ void ofono_cbs_notify(struct ofono_cbs *cbs, const unsigned char *pdu,
 
 out:
 	g_free(message);
-	g_slist_foreach(cbs_list, (GFunc)g_free, NULL);
-	g_slist_free(cbs_list);
+	g_slist_free_full(cbs_list, g_free);
 }
 
 static DBusMessage *cbs_get_properties(DBusConnection *conn,
@@ -337,8 +334,7 @@ static void cbs_set_topics_cb(const struct ofono_error *error, void *data)
 	char *topics;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		g_slist_foreach(cbs->new_topics, (GFunc)g_free, NULL);
-		g_slist_free(cbs->new_topics);
+		g_slist_free_full(cbs->new_topics, g_free);
 		cbs->new_topics = NULL;
 
 		DBG("Setting Cell Broadcast topics failed");
@@ -347,8 +343,7 @@ static void cbs_set_topics_cb(const struct ofono_error *error, void *data)
 		return;
 	}
 
-	g_slist_foreach(cbs->topics, (GFunc)g_free, NULL);
-	g_slist_free(cbs->topics);
+	g_slist_free_full(cbs->topics, g_free);
 	cbs->topics = cbs->new_topics;
 	cbs->new_topics = NULL;
 
@@ -560,25 +555,6 @@ static const GDBusSignalTable cbs_signals[] = {
 	{ }
 };
 
-int ofono_cbs_driver_register(const struct ofono_cbs_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	if (d->probe == NULL)
-		return -EINVAL;
-
-	g_drivers = g_slist_prepend(g_drivers, (void *) d);
-
-	return 0;
-}
-
-void ofono_cbs_driver_unregister(const struct ofono_cbs_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	g_drivers = g_slist_remove(g_drivers, (void *) d);
-}
-
 static void cbs_unregister(struct ofono_atom *atom)
 {
 	struct ofono_cbs *cbs = __ofono_atom_get_data(atom);
@@ -590,21 +566,18 @@ static void cbs_unregister(struct ofono_atom *atom)
 	ofono_modem_remove_interface(modem, OFONO_CELL_BROADCAST_INTERFACE);
 
 	if (cbs->topics) {
-		g_slist_foreach(cbs->topics, (GFunc) g_free, NULL);
-		g_slist_free(cbs->topics);
+		g_slist_free_full(cbs->topics, g_free);
 		cbs->topics = NULL;
 	}
 
 	if (cbs->new_topics) {
-		g_slist_foreach(cbs->new_topics, (GFunc) g_free, NULL);
-		g_slist_free(cbs->new_topics);
+		g_slist_free_full(cbs->new_topics, g_free);
 		cbs->new_topics = NULL;
 	}
 
 	if (cbs->efcbmid_length) {
 		cbs->efcbmid_length = 0;
-		g_slist_foreach(cbs->efcbmid_contents, (GFunc) g_free, NULL);
-		g_slist_free(cbs->efcbmid_contents);
+		g_slist_free_full(cbs->efcbmid_contents, g_free);
 		cbs->efcbmid_contents = NULL;
 	}
 
@@ -629,7 +602,7 @@ static void cbs_unregister(struct ofono_atom *atom)
 	if (cbs->settings) {
 		storage_close(cbs->imsi, SETTINGS_STORE, cbs->settings, TRUE);
 
-		g_free(cbs->imsi);
+		l_free(cbs->imsi);
 		cbs->imsi = NULL;
 		cbs->settings = NULL;
 	}
@@ -665,41 +638,9 @@ static void cbs_remove(struct ofono_atom *atom)
 	g_free(cbs);
 }
 
-struct ofono_cbs *ofono_cbs_create(struct ofono_modem *modem,
-					unsigned int vendor,
-					const char *driver,
-					void *data)
-{
-	struct ofono_cbs *cbs;
-	GSList *l;
-
-	if (driver == NULL)
-		return NULL;
-
-	cbs = g_try_new0(struct ofono_cbs, 1);
-
-	if (cbs == NULL)
-		return NULL;
-
-	cbs->assembly = cbs_assembly_new();
-	cbs->atom = __ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_CBS,
-						cbs_remove, cbs);
-
-	for (l = g_drivers; l; l = l->next) {
-		const struct ofono_cbs_driver *drv = l->data;
-
-		if (g_strcmp0(drv->name, driver))
-			continue;
-
-		if (drv->probe(cbs, vendor, data) < 0)
-			continue;
-
-		cbs->driver = drv;
-		break;
-	}
-
-	return cbs;
-}
+OFONO_DEFINE_ATOM_CREATE(cbs, OFONO_ATOM_TYPE_CBS, {
+	atom->assembly = cbs_assembly_new();
+})
 
 static void cbs_got_file_contents(struct ofono_cbs *cbs)
 {
@@ -729,15 +670,13 @@ static void cbs_got_file_contents(struct ofono_cbs *cbs)
 
 	if (cbs->efcbmi_length) {
 		cbs->efcbmi_length = 0;
-		g_slist_foreach(cbs->efcbmi_contents, (GFunc) g_free, NULL);
-		g_slist_free(cbs->efcbmi_contents);
+		g_slist_free_full(cbs->efcbmi_contents, g_free);
 		cbs->efcbmi_contents = NULL;
 	}
 
 	if (cbs->efcbmir_length) {
 		cbs->efcbmir_length = 0;
-		g_slist_foreach(cbs->efcbmir_contents, (GFunc) g_free, NULL);
-		g_slist_free(cbs->efcbmir_contents);
+		g_slist_free_full(cbs->efcbmir_contents, g_free);
 		cbs->efcbmir_contents = NULL;
 	}
 
@@ -907,8 +846,7 @@ static void cbs_efcbmid_changed(int id, void *userdata)
 
 	if (cbs->efcbmid_length) {
 		cbs->efcbmid_length = 0;
-		g_slist_foreach(cbs->efcbmid_contents, (GFunc) g_free, NULL);
-		g_slist_free(cbs->efcbmid_contents);
+		g_slist_free_full(cbs->efcbmid_contents, g_free);
 		cbs->efcbmid_contents = NULL;
 	}
 
@@ -930,7 +868,7 @@ static void cbs_got_imsi(struct ofono_cbs *cbs)
 	if (cbs->settings == NULL)
 		return;
 
-	cbs->imsi = g_strdup(imsi);
+	cbs->imsi = l_strdup(imsi);
 
 	cbs->topics = NULL;
 
@@ -1029,11 +967,14 @@ out:
 
 	/*
 	 * In order to minimize signal transmissions we wait about X seconds
-	 * before reseting the base station id.  The hope is that we receive
+	 * before resetting the base station id.  The hope is that we receive
 	 * another cell broadcast with the new base station name within
 	 * that time
 	 */
 	if (lac_changed || ci_changed) {
+		if(cbs->reset_source)
+			g_source_remove(cbs->reset_source);
+
 		cbs->reset_source =
 			g_timeout_add_seconds(3, reset_base_station_name, cbs);
 	}
