@@ -19,6 +19,7 @@
  *
  */
 
+#include <stdbool.h>
 #include <glib.h>
 
 #include <ofono/types.h>
@@ -158,11 +159,13 @@ enum context_status {
 
 const char *telephony_error_to_str(const struct ofono_error *error);
 
-gboolean valid_number_format(const char *number, int length);
-gboolean valid_phone_number_format(const char *number);
-gboolean valid_long_phone_number_format(const char *number);
+bool valid_number_format(const char *number, size_t length);
+bool valid_phone_number_format(const char *number);
+bool valid_long_phone_number_format(const char *number);
 const char *phone_number_to_string(const struct ofono_phone_number *ph);
-void string_to_phone_number(const char *str, struct ofono_phone_number *ph);
+
+void __string_to_phone_number(const char *str, struct ofono_phone_number *ph);
+int string_to_phone_number(const char *str, struct ofono_phone_number *ph);
 
 int mmi_service_code_to_bearer_class(int code);
 
@@ -185,8 +188,8 @@ gboolean is_valid_apn(const char *apn);
 const char *call_status_to_string(enum call_status status);
 
 const char *gprs_proto_to_string(enum ofono_gprs_proto proto);
-gboolean gprs_proto_from_string(const char *str, enum ofono_gprs_proto *proto);
+bool gprs_proto_from_string(const char *str, enum ofono_gprs_proto *proto);
 
 const char *gprs_auth_method_to_string(enum ofono_gprs_auth_method auth);
-gboolean gprs_auth_method_from_string(const char *str,
+bool gprs_auth_method_from_string(const char *str,
 					enum ofono_gprs_auth_method *auth);
